function BoardController(model, view, infoView, commentView, guessBar) {
  var guessMode;

  this.refresh = function() {
    view.refresh();
    commentView.refresh();
  }

  this.displayCoords = function(show) {
    view.modifyCoords(show);
  }

  this.setGuessMode = function(enabled) {
    guessBar.setAttribute("hidden", ! enabled);

    guessMode = enabled;
  }

  this.prev = function () {
    model.up();
    this.refresh();
  }

  this.next = function() {
    model.down();
    this.refresh();
  }

  this.fastforward = function() {
    for (var i = 0; i < 10; i++) {
      model.down();
    }

    this.refresh();
  }

  this.fastbackward = function() {
    for (var i = 0; i < 10; i++) {
      model.up();
    }

    this.refresh();
  }

  this.beginning = function() {
    while (model.up() != undefined) {
      // just loop
    }

    this.refresh();
  }

  this.end = function() {
    while (model.down() != undefined) {
      // just loop
    }

    this.refresh();
  }

  this.nextVariation = function() {
    var mode = FoxGo.prefs.getVariationMode();

    if (mode == "next") {
      model.nextVariation(false);
    } else {
      model.nextVariation(true);
    }

    this.refresh();
  }

  this.prevVariation = function() {
    var mode = FoxGo.prefs.getVariationMode();

    if (mode == "next") {
      model.prevVariation(false);
    } else {
      model.prevVariation(true);
    }

    this.refresh();
  }

  this.processClick = function(x, y) {
    var ax = x - 1;
    var ay = y - 1;

    if (! guessMode) {
      if (! model.hasVariation(ax, ay)) {
        model.addMove(ax, ay, false);
      } else {
        model.toVariation(ax, ay);
      }
    } else if (model.getNextMove() != undefined) {
      var next = model.getNextMove();
      var dx = ax - next.coords[0];
      var dy = ay - next.coords[1];

      var dist = Math.sqrt(dx * dx + dy * dy);
      var percent = Math.floor(((10 - dist) / 10) * 100);

      guessBar.setAttribute("value", percent);

      if (dist == 0) {
        model.toVariation(ax, ay);
      }
    }

    this.refresh();
  }

  this.flashMove = function(moveStr) {
      var pat = /([bw])(\d+)/i;
      var match = pat.exec(moveStr);
      var index = match[2] * 1 - 1;
      var node = model.getMove(index);
      var move = node.getMove();

      view.flashCoordinate(move.coords[0], move.coords[1]);
  }

  this.flashCoordinate = function(coordStr) {
    var charbase = "a".charCodeAt(0);
    var ival = "i".charCodeAt(0) - charbase;
    var pat = /([a-zA-Z]+)(\d+)/;
    var match = pat.exec(coordStr);

    var x = match[1].toLowerCase().charCodeAt(0) - charbase;
    var y = match[2] * 1 - 1;

    if (x > ival) {
      x--;
    }

    view.flashCoordinate(x, 18 - y);
  }

  this.initialize = function() {
    view.initialize(this);
    view.refresh();
    commentView.refresh();
    infoView.refresh();
  }

  this.newGame = function() {
    this.initialize();

    infoView.refreshGameMenu();
  }

  this.load = function(url) {
    model.load(url);
    this.initialize();

    infoView.refreshGameMenu();
  }

  this.changeGame = function(index) {
    model.setGame(index);

    this.initialize();
  }

  this.openURL = function() {
    var url = prompt("Enter URL");

    if (url != undefined) {
      this.load(url);
    }
  }

  this.openFile = function() {
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);

    fp.init(window, "Select SGF File", fp.modeOpen);
    fp.appendFilter("SGF File(*.sgf)", "*.sgf");
    fp.appendFilters(fp.filterAll);
    
    var showResult = fp.show();

    if(showResult == fp.returnOK) {
      var file = fp.file.path;

      FoxGo.log("URL " + file);
    } else {
      return;
    }

    var uriFixup = Components.classes['@mozilla.org/docshell/urifixup;1'].getService(Components.interfaces.nsIURIFixup);
    var url = uriFixup.createFixupURI(fp.file.path, uriFixup.FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP);    

    this.load(url.spec);
  }

  this.openTest = function() {
    this.load("http://frodo.dyn.gno.org/~brettk/test.sgf");
  }
}
