const hoshi_defs = { 19 : [[3, 3], [9, 3], [15, 3],
			   [3, 9], [9, 9], [15, 9],
			   [3, 15], [9, 15], [15, 15]],
		     13 : [[3, 3], [6, 3], [9, 3],
			   [3, 6], [6, 6], [9, 6],
			   [3, 9], [6, 9], [9, 9]],
                     9  : [[2, 2], [6, 2], 
			   [4, 4],
			   [2, 6], [6, 6]],
                     7  : [[3, 3]]};

function BoardView(model, grid, moveLabel, bPrisonerLabel, wPrisonerLabel) {
  var cells;
  var controller;

  function clearElement(elem) {
    var children = elem.childNodes;
    var length = children.length;

    for (var i = length - 1; i >= 0; i--) {
      elem.removeChild(children.item(i));
    }
  }

  function getHoshis(size) {
    var hoshis = hoshi_defs[model.size];

    if (hoshis != undefined) {
      return hoshis;
    } else {
      return new Array();
    }
  }

  // Draw the board grid
  this.initializeGrid = function() {
    var document = grid.ownerDocument;
    var children = grid.childNodes;
    var length = children.length;

    clearElement(grid);

    var size = model.size;
    var columns = document.createElement("columns");
    var rows = document.createElement("rows");    

    for (var x = 0; x < size + 2; x++) {
      var column = document.createElement("column");

      columns.appendChild(column);
    }

    cells = new Array();

    for (var y = 0; y < size + 2; y++) {
      var row = document.createElement("row");

      cells[y] = new Array();

      rows.appendChild(row);

      for (var x = 0; x < size + 2; x++) {
        var stack = document.createElement("stack");

        cells[y][x] = stack;
        row.appendChild(stack);

	if ((x < 1) || (x > model.size) ||
	    (y < 1) || (y > model.size)) {
	  continue;
	}

	var image = document.createElement("image");

        cells[y][x].image = image;
        cells[y][x].markers = new Array();

        setCell(x - 1, y - 1, BOARD_EMPTY);

        stack.appendChild(image);

        eval("stack.addEventListener('click'," +
             "                       function(evt) {" +
             "                         controller.processClick(" + x + ", " + y + ");" +
             "                       }," +
             "                       true);"); 
      }
    }

    grid.appendChild(columns);
    grid.appendChild(rows);
  }

  function setCell(x, y, type) {
    var stack = cells[y + 1][x + 1];
    var id;
    var x_end = model.size - 1;
    var y_end = model.size - 1;

    var hoshis = getHoshis(model.size);
    var hoshi = hoshis.filter(function (coords) {
				return ((coords[0] == x) &&
					(coords[1] == y));
			      });

    if (type == BOARD_EMPTY) {
      if ((x == 0) && (y == 0)) {
        id = "foxgo-cell-ul";
      } else if ((x < x_end) && (y == 0)) {
        id = "foxgo-cell-u";
      } else if ((x == x_end) && (y == 0)) {
        id = "foxgo-cell-ur";
      } else if ((x == 0) && (y < y_end)) {
        id = "foxgo-cell-l";
      } else if ((x == x_end) && (y < y_end)) {
        id = "foxgo-cell-r";
      } else if ((x == 0) && (y == y_end)) {
        id = "foxgo-cell-dl";
      } else if ((x < x_end) && (y == y_end)) {
        id = "foxgo-cell-d";
      } else if ((x == x_end) && (y == y_end)) {
        id = "foxgo-cell-dr";
      } else if (hoshi.length > 0) {
        id = "foxgo-cell-hoshi";
      } else {
        id = "foxgo-cell-empty";
      }
    } else if (type == BOARD_BLACK) {
      id = "foxgo-cell-black";
    } else if (type == BOARD_WHITE) {
      id = "foxgo-cell-white";
    }

    stack.markers.forEach(function (m) {
      stack.removeChild(m);
    });

    stack.markers = new Array();

    stack.image.setAttribute("id", id);
  }
  
  function addMarker(x, y, id) {
    var stack = cells[y + 1][x + 1];
    var image = document.createElement("image");

    image.setAttribute("id", id);

    stack.markers.push(image);
    stack.appendChild(image);                         
  }

  function addLabel(x, y, str) {
    var stack = cells[y + 1][x + 1];
    var hbox = document.createElement("hbox");
    var label = document.createElement("label");

    hbox.setAttribute("minwidth", stack.boxObject.width);
    hbox.setAttribute("maxwidth", stack.boxObject.width);
    hbox.setAttribute("width", stack.boxObject.width);
    hbox.setAttribute("pack", "center");
    
    label.setAttribute("id", "foxgo-cell-label");
    label.setAttribute("flex", 1);
    label.setAttribute("crop", "end");
    label.setAttribute("value", str);
    label.setAttribute("tooltiptext", str);

    hbox.appendChild(label);
    stack.appendChild(hbox);

    stack.markers.push(hbox);
  }

  this.modifyCoords = function(enabled) {
    function boxup(x, text) {
      var box;
      var label = document.createElement("label");

      if (x) {
	box = document.createElement("hbox");
      } else {
	box = document.createElement("vbox");
      }

      box.setAttribute("pack", "center");
      label.setAttribute("value", text);
      label.setAttribute("id", "foxgo-coordinate-label");

      box.appendChild(label);

      return box;
    }

    var base = "A".charCodeAt(0);
    var letter_i = "I".charCodeAt(0);

    for (var i = 0; i < model.size; i++) {
      var top_cell = cells[0][i + 1];
      var bottom_cell = cells[model.size + 1][i + 1];

      var left_cell = cells[i + 1][0];
      var right_cell = cells[i + 1][model.size + 1];

      if (enabled) {
	var x_code = base + i;
	var y_code = i;

	if (x_code >= letter_i) {
	  x_code++;
	}

	var x_str = String.fromCharCode(x_code);
	var y_str = model.size - y_code;

	top_cell.appendChild(boxup(true, x_str));
	bottom_cell.appendChild(boxup(true, x_str));
	left_cell.appendChild(boxup(false, y_str));
	right_cell.appendChild(boxup(false, y_str));
      } else {
	clearElement(top_cell);
	clearElement(bottom_cell);

	clearElement(left_cell);
	clearElement(right_cell);
      }
    }
  }

  this.refresh = function() {
    var x, y;

    for (y = 0; y < model.size; y++) {
      for (x = 0; x < model.size; x++) {
	var obj = model.objectAt(x, y);

        if (obj == undefined) {
          setCell(x, y, BOARD_EMPTY);
        } else {
          setCell(x, y, obj.colour);
        }
      }
    }

    var varMode = FoxGo.prefs.getVariationMode();

    if (varMode != "none") {
      var variations = model.getVariations(varMode == "current");
      var smallVar = FoxGo.prefs.useSmallVarMarker();
      var varType = (smallVar ? "mini" : "var");
      
      variations.forEach(function (variation) {
        var id;

        if (variation.colour == BOARD_BLACK) {
          id = "foxgo-cell-black-" + varType;
        } else if (variation.colour == BOARD_WHITE) {
          id = "foxgo-cell-white-" + varType;
        } else {
        }

        addMarker(variation.coords[0],
                  variation.coords[1],
                  id);
      });

      if ((variations.length > 0) && (varMode == "next")) {
	var node = model.getNextMove();

	addMarker(node.coords[0],
		  node.coords[1],
		  "foxgo-current-variation-marker");
      }
    }

    if (FoxGo.prefs.showCurrentMoveMarker()) {
      var cur = model.getCurrentMove();
      
      if (cur != undefined) {
	var colour;
	
	if (cur.colour == BOARD_BLACK) {
	  colour = "black";
	}
	
	if (cur.colour == BOARD_WHITE) {
	  colour = "white";
	}
	
	addMarker(cur.coords[0],
		  cur.coords[1],
		  "foxgo-cell-circle-" + colour);
      }
    };

    var classMap = { "TR" : "triangle",
                     "SQ" : "square",
                     "CR" : "circle",
                     "MA" : "marked",
                     "SL" : "selected" };

    var markers = model.getMarkers();

    markers["TW"].forEach(function (coords) {
			    addMarker(coords[0],
				      coords[1],
				      "foxgo-cell-territory-white");
			  });
    
    markers["TB"].forEach(function (coords) {
			    addMarker(coords[0],
				      coords[1],
				      "foxgo-cell-territory-black");
			  });

    for (marker in classMap) {
      markers[marker].forEach(function (coords) {
        var cell = model.objectAt(coords[0], coords[1]);
        var colour;
			    
        if (cell == undefined) {
	  colour = "empty";
        } else if (cell.colour == BOARD_BLACK) {
          colour = "black";
        } else if (cell.colour == BOARD_WHITE) {
          colour = "white";
        }
			    
        addMarker(coords[0],
                  coords[1],
                  "foxgo-cell-" + classMap[marker] + "-" + colour);
      });
    }

    var labels = model.getLabels();

    labels.forEach(function (lb) {
		     addLabel(lb[0][0],
			      lb[0][1],
			      lb[1]);
		   });

    moveLabel.setAttribute("value", model.getMoveNumber());

    bPrisonerLabel.setAttribute("value", model.getBlackPrisoners());
    wPrisonerLabel.setAttribute("value", model.getWhitePrisoners());
  }

  this.flashCoordinate = function(x, y) {
    var stack = cells[y + 1][x + 1];
    var image = document.createElement("image");
    var count = 0;
    var id;
    var func;

    image.setAttribute("id", "foxgo-cell-flash");
    stack.appendChild(image);

    func = function() {
      if ((count % 2) == 0) {
	image.setAttribute("hidden", true);
      } else {
	image.setAttribute("hidden", false);			  
      }

      if (++count > 2) {
	window.clearTimeout(id);
	stack.removeChild(image);
      }
    };

    id = window.setInterval(func, 100);
  }

  this.initialize = function(control) {
    controller = control;

    this.initializeGrid();
    this.modifyCoords(FoxGo.prefs.showCoordinates());
  }
}
