function Prefs(window, controller) {  
  var prefix = "foxgo.";

  this.initializeMenu = function() {
    var varMode = this.getVariationMode();
    var showCurrentMarker = this.showCurrentMoveMarker();
    var smallMarker = this.useSmallVarMarker();
    var coords = this.showCoordinates();
    var guessMode = this.guessMode();
    var item;

    if (varMode == "current") {
      item = window.getElementById("foxgo-varitem-current-move");     
    } else if (varMode == "next") {
      item = window.getElementById("foxgo-varitem-next-move");     
    } else if (varMode == "none") {
      item = window.getElementById("foxgo-varitem-none");     
    }

    item.setAttribute("checked", true);

    item = window.getElementById("foxgo-viewitem-current-move-marker");
    item.setAttribute("checked", showCurrentMarker);

    item = window.getElementById("foxgo-viewitem-small-var-marker");
    item.setAttribute("checked", smallMarker);

    item = window.getElementById("foxgo-viewitem-coordinates");
    item.setAttribute("checked", coords);

    item = window.getElementById("foxgo-guess-mode");
    item.setAttribute("checked", guessMode);

    controller.setGuessMode(guessMode);
  }

  this.setVariationMode = function(mode) {
    this.setPrefValue(prefix + "variations.mode",
		      "str", 
		      mode);
    controller.refresh();
  }

  this.getVariationMode = function() {
    return this.getPrefValue(prefix + "variations.mode",
			     "str",
			     "current");
  }

  this.toggleUseSmallVarMarker = function() {
    this.setPrefValue(prefix + "variations.use_small_marker",
		      "bool",
		      ! this.useSmallVarMarker());
    controller.refresh();
  }

  this.useSmallVarMarker = function() {
    return this.getPrefValue(prefix + "variations.use_small_marker",
			     "bool",
			     false);
  }

  this.toggleShowCurrentMoveMarker = function() {
    this.setPrefValue(prefix + "show_move_marker",
		      "bool",
		      ! this.showCurrentMoveMarker());
    controller.refresh();
  }

  this.showCurrentMoveMarker = function() {
    return this.getPrefValue(prefix + "show_move_marker",
			     "bool",
			     true);
  }

  this.toggleShowCoordinates = function() {
    var val = ! this.showCoordinates();

    this.setPrefValue(prefix + "show_coords",
		      "bool",
		      val);

    FoxGo.controller.displayCoords(val);
  }

  this.showCoordinates = function() {
    return this.getPrefValue(prefix + "show_coords",
			     "bool",
			     true);
  }

  this.toggleGuessMode = function() {
    var val = ! this.guessMode();

    this.setPrefValue(prefix + "guess_mode",
		      "bool",
		      val);

    FoxGo.controller.setGuessMode(val);
  }

  this.guessMode = function() {
    return this.getPrefValue(prefix + "guess_mode",
			     "bool",
			     false);
  }

  this.setPrefValue = function(aPrefString, aPrefType, aValue) {
    var nsISupportsString = Components.interfaces.nsISupportsString;
    var xpPref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

    var prefType = xpPref.getPrefType(aPrefString);

    try {
      switch(aPrefType) {
	case "wstr": {
	  var string = Components.classes['@mozilla.org/supports-string;1'].createInstance(nsISupportsString);
	  string.data = aValue;
	  return xpPref.setComplexValue(aPrefString, nsISupportsString, string);
	  break;
        }

	case "str": {
	  return xpPref.setCharPref(aPrefString, aValue);
	  break;
        }

	case "int": {
	  aValue = parseInt(aValue);
	  return xpPref.setIntPref(aPrefString, aValue);
	  break;
        }

	case "bool":
	default: {
	  if(typeof(aValue) == "string") {
	    aValue = (aValue == "true");
	  }
	  return xpPref.setBoolPref(aPrefString, aValue);
        }
      }
    } catch(e) {
      FoxGo.log("" + e);
    }
    return null;
  }

  this.getPrefValue = function(aPrefString, aPrefType, aDefault) {
    var nsISupportsString = Components.interfaces.nsISupportsString;
    var xpPref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

    if(xpPref.getPrefType(aPrefString) == xpPref.PREF_INVALID) {
      return aDefault;
    }
    try {
      switch (aPrefType) {
	case "wstr": {
	  return xpPref.getComplexValue(aPrefString, nsISupportsString).data;
	  break;
        }

	case "str": {
	  return xpPref.getCharPref(aPrefString).toString();
	  break;
        }

	case "int": {
	  return xpPref.getIntPref(aPrefString);
	  break;
        }

	case "bool": 
	default: {
	  return xpPref.getBoolPref(aPrefString);
	  break;
        }
      }
    } catch(e) {
      FoxGo.log("" + e);
    }
    return aDefault;
  }
}
