function Move(coords, colour, pass) {
  this.coords = coords;
  this.colour = colour;
  this.pass = pass;
}

function GameNode() {
  this.nodes = new Array(); 
  this.properties = new Object();
  this.variation = 0;

  function convertCoordStr(str) {
    var charbase = "a".charCodeAt(0);

    return new Array(str.charCodeAt(0) - charbase,
		     str.charCodeAt(1) - charbase);
  }
  
  this.addNode = function(node) {
    node.parent = this;
    this.nodes.push(node);
  }

  this.addProperty = function(name, val) {
    var values = this.properties[name];

    if (values == undefined) {
      values = new Array();
    }

    values.push(val);

    this.properties[name] = values;
  }

  this.hasProperty = function(name) {
    return (this.properties[name] != undefined);
  }

  this.getProperty = function(name) {
    var val = this.properties[name];

    if (val != undefined) {
      return val[0];
    } else {
      return undefined;
    }
  }

  this.getAllProperty = function(name) {
    var val = this.properties[name];

    if (val != undefined) {
      return val;
    } else {
      return new Array();
    }
  }

  this.getCompoundProperty = function(name) {
    var vals = this.properties[name];
    var res = new Array();

    if (vals == undefined) {
      return res;
    }

    vals.forEach(function (val) {
		   res.push(val.split(":"));
		 });

    return res;
  }

  this.getPropertyAsCoords = function(name) {
    var vals = this.getCompoundProperty(name);
    var res = new Array();

    vals.forEach(function (val) {
		   for (var i = 0; i < val.length; i++) {
		     val[i] = convertCoordStr(val[i]);
		   }

		   if (val.length == 1) {
		     res.push(val[0]);
		   } else {
		     var start = val[0];
		     var end = val[1];
		     
		     for (var x = start[0]; x <= end[0]; x++) {
		       for (var y = start[1]; y <= end[1]; y++) {
			 res.push(new Array(x, y));
		       }
		     }
		   }
		});

    return res;
  }

  this.getPropertyAsLabel = function(name) {
    var vals = this.getCompoundProperty(name);
    var res = new Array();

    vals.forEach(function (val) {
		   var coords = convertCoordStr(val[0]);

		   res.push(new Array(coords, val[1]));
		 });

    return res;
  }

  this.addPropertyCoords = function(name, x, y) {
    var coordStr = "";
    var base = "a".charCodeAt(0);

    coordStr += String.fromCharCode(base + x);
    coordStr += String.fromCharCode(base + y);

    this.addProperty(name, coordStr);
  }

  this.getMove = function() {
    var c;
    var colour;

    if (this.hasProperty("B")) {
      c = this.getPropertyAsCoords("B");
      colour = BOARD_BLACK;
    } else if (this.hasProperty("W")) {
      c = this.getPropertyAsCoords("W");
      colour = BOARD_WHITE;
    } else {
      return undefined;
    }

    if (c.length > 0) {
      return new Move(c[0], colour, false);
    } else {
      return new Move(undefined, colour, true);
    }
  }

  this.dumpNode = function(level) {
    var pad = "";

    for (var i = 0; i < level; i++) {
      pad += " ";
    }

    for (var p in this.properties) {
      FoxGo.log(pad + " " + p + "=" + this.properties[p]);
    }
  }

  this.dumpInternal = function(level) {
    var pad = "";

    for (var i = 0; i < level; i++) {
      pad += " ";
    }

    FoxGo.log(pad + "(\n");

    dumpNode(level);

    for (var i = 0; i < this.nodes.length; i++) {
      this.nodes[i].dumpInternal(level + 2);
    }

    FoxGo.log(pad + ")\n");
  }

  this.dump = function() {
    this.dumpInternal(0);
  }
}

function SGF(url) {
  var text;
  var pos = 0;

  var NORMAL = 0;
  var ESCAPE = 1;
  var DONE = 2;

  var whitespace = /[\t\f\v\0]/;    
  var all_whitespace = /\s/;

  function nextChar() {
    var c = text[pos++];

    if (c == '\r') {
      c = '\n';

      if (text[pos++] != '\n') {
	rewind();
      }
    } else if (c == '\n') {
      if (text[pos++] != '\r') {
        rewind();
      }
    }

    return c;
  }

  function rewind() {
    pos--;
  }

  function SGFException(message) {
    this.position = pos;
    this.message = message;
  }

  function loadURL() {
    var req = new XMLHttpRequest();
    
    req.open('GET', url, false);
    req.send(null);

    if ((req.status != 0) && (req.status != 200)) {
      alert('Error loading ' + url);
      
      for (p in req) { FoxGo.log(p + "=" + req[p]); }

      return false;
    }

    text = req.responseText;
  }

  function parseGameTree() {
    var root;
    var curNode;

    while(true) {
      var c = nextChar();
      var node;

      if (c == ';') {
	node = parseNode();

	if (root == undefined) {
	  root = node;
	} else {
	  curNode.addNode(node);
	}

	curNode = node;
      } else if (c == '(') {
	curNode.addNode(parseGameTree());
      } else if (c == ')') {
	break;
      } else if (! all_whitespace.exec(c)) {
        continue;
      }
    }

    return root;
  }

  function parseNode() {
    var node = new GameNode();

    while (true) {
      var pname = parsePropertyName();

      if (pname == undefined) {
	return node;
      }

      while ((value = parsePropertyValue()) != undefined) {
	if (value != '') {
	  node.addProperty(pname, value);
	}
      }
    }
  }

  var test = 0;

  function parsePropertyName() {
    var pat = /[A-Za-z]/;
    var str = "";
    var c = nextChar();
    var start = "A".charCodeAt(0);
    var end = "Z".charCodeAt(0);
    
    while (all_whitespace.exec(c)) {
      c = nextChar();
    }

    while (pat.exec(c)) {
      var code = c.charCodeAt(0);

      if ((code >= start) && (code <= end)) {
        str += c;      
      }

      c = nextChar();
    }
    
    rewind();

    if (str.length == 0) {
      return undefined;
    } else {
      return str;
    }
  }

  function parsePropertyValue() {
    var c = nextChar();
    var value = "";
    var state = NORMAL;

    while (all_whitespace.exec(c)) {
      c = nextChar();
    }

    if (c != '[') {
      rewind();
      return undefined;
    }

    while (state != DONE) {
      c = nextChar();

      switch (state) {
      case NORMAL:
	if (c == '\\') {
	  state = ESCAPE;
	} else if (c == ']') {
	  state = DONE;
	} else if (whitespace.exec(c)) {
	  value += ' ';
	} else {
	  value += c;
	}

	break;

      case ESCAPE:
	if (whitespace.exec(c)) {
	  value += ' ';
	} else if (c != '\n') {
	  value += c;
	}

	state = NORMAL;

	break;
      }
    }

    return value;
  }

  this.read = function() {
    loadURL();

    var roots = new Array();

    while (pos < text.length) {
      var c = nextChar();

      if (c == '(') {
	roots.push(parseGameTree());
      } else if (! all_whitespace.exec(c)) {
	// do error stuff here
      }
    }

    return roots;
  }
}
